@@ -9,12 +9,13 @@ Arguments:
 
 * `wait` (float): number of seconds to wait between retries (default `1.0`)
 
+* `expected_matches` (int): number of matches that the expression should match (default `1`)
+
 Instances of this task is created with step expression:
 
-* [`step_task_request_text_with_name_to_endpoint_until`](/grizzly/usage/steps/scenario/tasks/#step_task_request_text_with_name_to_endpoint_until)
+* [`step_task_request_text_with_name_to_endpoint_until`](/grizzly/framework/usage/steps/scenario/tasks/#step_task_request_text_with_name_to_endpoint_until)
 '''
 from typing import TYPE_CHECKING, Callable, Any, Type, List, Optional, cast
-from dataclasses import dataclass, field
 from time import perf_counter as time
 
 from jinja2 import Template
@@ -23,25 +24,34 @@ from grizzly_extras.transformer import Transformer, TransformerContentType, Tran
 from grizzly_extras.arguments import get_unsupported_arguments, parse_arguments, split_value
 
 from ..context import GrizzlyContext
-from ..types import GrizzlyTask
 from .request import RequestTask
+from . import GrizzlyTask, template
 
 if TYPE_CHECKING:  # pragma: no cover
     from ..scenarios import GrizzlyScenario
 
 
-@dataclass
+@template('condition', 'expression')
 class UntilRequestTask(GrizzlyTask):
     request: RequestTask
     condition: str
 
-    transform: Optional[Type[Transformer]] = field(init=False)
-    matcher: Callable[[Any], List[str]] = field(init=False)
+    transform: Optional[Type[Transformer]]
+    matcher: Callable[[Any], List[str]]
 
-    retries: int = field(init=False, default=3)
-    wait: float = field(init=False, default=1.0)
+    retries: int
+    wait: float
+    expected_matches: int
+
+    def __init__(self, request: RequestTask, condition: str) -> None:
+        super().__init__()
+
+        self.request = request
+        self.condition = condition
+        self.retries = 3
+        self.wait = 1.0
+        self.expected_matches = 1
 
-    def __post_init__(self) -> None:
         if self.request.response.content_type == TransformerContentType.UNDEFINED:
             raise ValueError('content type must be specified for request')
 
@@ -56,13 +66,14 @@ class UntilRequestTask(GrizzlyTask):
 
             arguments = parse_arguments(until_arguments)
 
-            unsupported_arguments = get_unsupported_arguments(['retries', 'wait'], arguments)
+            unsupported_arguments = get_unsupported_arguments(['retries', 'wait', 'expected_matches'], arguments)
 
             if len(unsupported_arguments) > 0:
                 raise ValueError(f'unsupported arguments {", ".join(unsupported_arguments)}')
 
             self.retries = int(arguments.get('retries', self.retries))
             self.wait = float(arguments.get('wait', self.wait))
+            self.expected_matches = int(arguments.get('expected_matches', '1'))
 
             if self.retries < 1:
                 raise ValueError('retries argument cannot be less than 1')
@@ -70,18 +81,15 @@ class UntilRequestTask(GrizzlyTask):
             if self.wait < 0.1:
                 raise ValueError('wait argument cannot be less than 0.1 seconds')
 
-    def implementation(self) -> Callable[['GrizzlyScenario'], Any]:
+    def __call__(self) -> Callable[['GrizzlyScenario'], Any]:
         if self.transform is None:
             raise TypeError(f'could not find a transformer for {self.request.response.content_type.name}')
 
         transform = cast(Transformer, self.transform)
 
-        def _implementation(parent: 'GrizzlyScenario') -> Any:
-            task_name = f'{self.request.scenario.identifier} {self.request.name}, w={self.wait}s, r={self.retries}'
-            if '{{' in self.condition and '}}' in self.condition:
-                condition_rendered = Template(self.condition).render(**parent.user._context['variables'])
-            else:
-                condition_rendered = self.condition
+        def task(parent: 'GrizzlyScenario') -> Any:
+            task_name = f'{self.request.scenario.identifier} {self.request.name}, w={self.wait}s, r={self.retries}, em={self.expected_matches}'
+            condition_rendered = parent.render(self.condition)
 
             if not transform.validate(condition_rendered):
                 raise RuntimeError(f'{condition_rendered} is not a valid expression for {self.request.response.content_type.name}')
@@ -114,7 +122,7 @@ class UntilRequestTask(GrizzlyTask):
                         number_of_matches = 0
                         parent.logger.error(f'{task_name}: loop retry={retry}', exc_info=True)
 
-                    if number_of_matches == 1:
+                    if number_of_matches == self.expected_matches:
                         break
                     else:
                         retry += 1
@@ -124,9 +132,9 @@ class UntilRequestTask(GrizzlyTask):
 
             response_time = int((time() - start) * 1000)
 
-            if number_of_matches == 1:
+            if number_of_matches == self.expected_matches:
                 exception = None
-            elif exception is None and number_of_matches != 1:
+            elif exception is None and number_of_matches != self.expected_matches:
                 exception = RuntimeError((
                     f'found {number_of_matches} matching values for {condition_rendered} in payload '
                     f'after {retry} retries and {response_time} milliseconds'
@@ -144,4 +152,4 @@ class UntilRequestTask(GrizzlyTask):
             if exception is not None and self.request.scenario.failure_exception is not None:
                 raise self.request.scenario.failure_exception()
 
-        return _implementation
+        return task
