@@ -2,7 +2,7 @@
 import logging
 
 
-logger = logging.getLogger(__name__)
+logger: logging.Logger = logging.getLogger(__name__)
 
 
 from .restapi import RestApiUser
