@@ -1,19 +1,20 @@
-from abc import abstractmethod
+import logging
+
 from os import environ, path
 from typing import Any, Dict, Tuple, Optional, Set, cast
 from logging import Logger
+from abc import abstractmethod
 
 from jinja2 import Template
 from locust.exception import StopUser
 from locust.user.users import User
+from locust.user.task import LOCUST_STATE_RUNNING
 from locust.env import Environment
 
-from grizzly.context import GrizzlyContextScenario
-
-from ...types import GrizzlyResponse
+from ...context import GrizzlyContextScenario
+from ...types import GrizzlyResponse, ScenarioState
 from ...tasks import RequestTask
 from ...utils import merge_dicts
-from ..import logger as user_logger
 from . import FileRequests
 
 
@@ -24,6 +25,8 @@ class GrizzlyUser(User):
     }
     _scenario: GrizzlyContextScenario
 
+    _scenario_state: Optional[ScenarioState]
+
     __dependencies__: Set[str] = set()
 
     request_name_map: Dict[str, str] = {
@@ -31,7 +34,7 @@ class GrizzlyUser(User):
         'HELLO': 'HELO',
     }
 
-    logger: Logger = user_logger
+    logger: Logger
 
     weight: int = 1
 
@@ -40,10 +43,31 @@ class GrizzlyUser(User):
 
         self._context_root = environ.get('GRIZZLY_CONTEXT_ROOT', '.')
         self._context = merge_dicts({}, GrizzlyUser._context)
+        self.logger = logging.getLogger(f'{self.__class__.__name__}/{id(self)}')
+        self._scenario_state = None
+
+    @property
+    def scenario_state(self) -> Optional[ScenarioState]:
+        return self._scenario_state
+
+    @scenario_state.setter
+    def scenario_state(self, value: ScenarioState) -> None:
+        old_state = self._scenario_state
+        self._scenario_state = value
+        self.logger.debug(f'scenario state={old_state} -> {value}')
+
+    def stop(self, force: bool = False) -> bool:
+        if not force:
+            self.logger.debug('stop scenarios before stopping user')
+            self.scenario_state = ScenarioState.STOPPING
+            self._state = LOCUST_STATE_RUNNING
+            return False
+        else:
+            return cast(bool, super().stop(force=force))
 
     @abstractmethod
     def request(self, request: RequestTask) -> GrizzlyResponse:
-        raise NotImplementedError(f'{self.__class__.__name__} has not implemented request(RequestTask)')
+        raise NotImplementedError(f'{self.__class__.__name__} has not implemented request')
 
     def get_request_method(self, request: RequestTask) -> str:
         return self.request_name_map.get(request.method.name, request.method.name[:4])
@@ -95,9 +119,9 @@ class GrizzlyUser(User):
         self._context = merge_dicts(self._context, context)
 
     def set_context_variable(self, variable: str, value: Any) -> None:
-        old_value = cast(Dict[str, Any], self._context['variables'])[variable] if variable in cast(Dict[str, Any], self._context['variables']) else None
+        old_value = self._context['variables'].get(variable, None)
         self._context['variables'][variable] = value
-        self.logger.debug(f'context: {variable=}, value: {old_value} -> {value}')
+        self.logger.debug(f'context {variable=}, value={old_value} -> {value}')
 
     @property
     def context_variables(self) -> Dict[str, Any]:
